import _extends from "@babel/runtime/helpers/esm/extends";
import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
import { Children, cloneElement, useEffect, useState, useRef, forwardRef } from 'react';
import PropTypes from 'prop-types';
import { useControlledState } from '@s-ui/react-hooks';
import loadable from '@loadable/component';
import { PLACEMENTS, TRIGGERS } from './config';
import { jsx as _jsx } from "react/jsx-runtime";
import { jsxs as _jsxs } from "react/jsx-runtime";
import { Fragment as _Fragment } from "react/jsx-runtime";
var BASE_CLASS = 'sui-AtomPopover';
var CLASS_INNER = BASE_CLASS + "-inner";
var PREFIX_PLACEMENT = BASE_CLASS + "-";
var DEFAULT_OFFSET = 'auto,4px';
var DEFAULT_TRIGGER = TRIGGERS.LEGACY;
var DEFAULT_DELAY = 0; // https://github.com/reactstrap/reactstrap/blob/7.1.0/src/TooltipPopoverWrapper.js

var Popover = loadable(function () {
  return import('reactstrap/lib/Popover');
}, {
  ssr: true
});

var adaptDeprecatedProps = function adaptDeprecatedProps(_ref) {
  var showPopover = _ref.showPopover,
      isVisible = _ref.isVisible,
      props = _objectWithoutPropertiesLoose(_ref, ["showPopover", "isVisible"]);

  return _extends({
    isVisible: isVisible === undefined ? showPopover : isVisible
  }, props);
};

var AtomPopover = /*#__PURE__*/forwardRef(function (props, outRef) {
  var _adaptDeprecatedProps = adaptDeprecatedProps(props),
      children = _adaptDeprecatedProps.children,
      closeIcon = _adaptDeprecatedProps.closeIcon,
      content = _adaptDeprecatedProps.content,
      _adaptDeprecatedProps2 = _adaptDeprecatedProps.onClose,
      onClose = _adaptDeprecatedProps2 === void 0 ? function () {} : _adaptDeprecatedProps2,
      _adaptDeprecatedProps3 = _adaptDeprecatedProps.onOpen,
      onOpen = _adaptDeprecatedProps3 === void 0 ? function () {} : _adaptDeprecatedProps3,
      _adaptDeprecatedProps4 = _adaptDeprecatedProps.placement,
      placement = _adaptDeprecatedProps4 === void 0 ? PLACEMENTS.BOTTOM : _adaptDeprecatedProps4,
      isVisible = _adaptDeprecatedProps.isVisible,
      defaultIsVisible = _adaptDeprecatedProps.defaultIsVisible,
      _adaptDeprecatedProps5 = _adaptDeprecatedProps.hideArrow,
      hideArrow = _adaptDeprecatedProps5 === void 0 ? true : _adaptDeprecatedProps5,
      _adaptDeprecatedProps6 = _adaptDeprecatedProps.trigger,
      trigger = _adaptDeprecatedProps6 === void 0 ? DEFAULT_TRIGGER : _adaptDeprecatedProps6;

  var targetRef = useRef();

  var _useState = useState(),
      ready = _useState[0],
      setReady = _useState[1];

  var _useControlledState = useControlledState(isVisible, defaultIsVisible),
      isVisibleState = _useControlledState[0],
      setIsVisibleState = _useControlledState[1];

  useEffect(function () {
    setReady(targetRef.current);
  }, [targetRef, setReady]);

  var extendChildren = function extendChildren() {
    var onClick = function onClick(onClickHandler) {
      if (onClickHandler === void 0) {
        onClickHandler = function onClickHandler() {
          return null;
        };
      }

      return function (ev) {
        onOpen();
        setIsVisibleState(true);
        onClickHandler(ev);
      };
    };

    var childrenOnly = typeof children === 'string' ? [/*#__PURE__*/_jsx("span", {
      children: children
    }, 1)] : Children.only(children);
    var response = Children.map(childrenOnly, function (child, key) {
      var attrs = {
        onClick: onClick(child.props.onClick)
      };

      attrs.ref = function (node) {
        ;
        [child.ref, targetRef].forEach(function (ref) {
          if (typeof ref === 'function') {
            ref(node);
          } else if (ref !== null) {
            ref.current = node;
          }
        });
      };

      attrs.key = key;
      return /*#__PURE__*/cloneElement(child, attrs);
    });
    return response;
  };

  var handleToggle = function handleToggle(ev) {
    setIsVisibleState(!isVisibleState);
    onClose(ev);
  };

  return /*#__PURE__*/_jsxs(_Fragment, {
    children: [extendChildren(), isVisibleState && ready && /*#__PURE__*/_jsxs(Popover, {
      className: BASE_CLASS,
      delay: DEFAULT_DELAY,
      innerClassName: CLASS_INNER,
      hideArrow: hideArrow,
      arrowClassName: BASE_CLASS + "-arrow",
      isOpen: isVisibleState,
      offset: DEFAULT_OFFSET,
      placement: placement,
      placementPrefix: PREFIX_PLACEMENT,
      target: targetRef.current,
      toggle: handleToggle,
      trigger: Array.isArray(trigger) ? trigger.join(' ') : trigger,
      innerRef: outRef,
      children: [closeIcon && /*#__PURE__*/_jsx("div", {
        className: BASE_CLASS + "-closeIcon",
        onClick: handleToggle,
        children: closeIcon
      }), content]
    })]
  });
});
AtomPopover.displayName = 'AtomPopover';
export { PLACEMENTS as atomPopoverPositions };
export { TRIGGERS as atomPopoverTriggers };
export default AtomPopover;